<?php
namespace PHPFusion\Roadmap;

class Roadmaps extends RoadmapServer {
    private static $instance = NULL;
    public $context = [];

    protected function __construct() {
        parent::__construct();
    }

    public static function getInstance() {
        if (self::$instance === NULL) {
            self::$instance = new static();
        }
       return self::$instance;
    }

    public function startRoadmap() {
        if ( check_get( 'file_id' ) && get( 'file_id', FILTER_VALIDATE_INT ) ) {
            $res = 0;
            $result = dbquery( "SELECT * FROM " . DB_ROADMAP . " WHERE item_id = :changelogid", [':changelogid' => get( 'file_id' )] );
            $data = dbarray( $result );
            if ( dbrows( $result ) > 0 && checkgroup( $data['item_access'] ) ) {
                dbquery( "UPDATE " . DB_ROADMAP . " SET item_download_count = item_download_count+1 WHERE item_id = :getfileid", [':getfileid' => get( 'file_id' )] );
                if ( !empty( $data['item_download'] ) && file_exists( ROADMAP_FILES . $data['item_download'] ) ) {
                    $res = 1;
                    require_once INCLUDES . "class.httpdownload.php";
                    ob_end_clean();
                    $object = new httpdownload;
                    $object->set_byfile( ROADMAP_FILES . $data['item_download'] );
                    $object->use_resume = TRUE;
                    $object->download();
                    exit;
                }
            }
            if ( $res == 0 ) {
                redirect( ROADMAP_PATH . "roadmap.php" );
            }
        }

        switch ( $this->sections ) {
            case 'empty':
                $context = self::rmpDefault( 0 );
                roadmap( $context );
                break;
            case 'new':
                $context = self::rmpDefault( 1 );
                roadmap( $context );
                break;
            case 'investigation':
                $context = self::rmpDefault( 2 );
                roadmap( $context );
                break;
            case 'rejected':
                $context = self::rmpDefault( 3 );
                roadmap( $context );
                break;
            case 'accepted':
                $context = self::rmpDefault( 4 );
                roadmap( $context );
                break;
            case 'development':
                $context = self::rmpDefault( 5 );
                roadmap( $context );
                break;
            case 'finished':
                $context = self::rmpDefault( 6 );
                roadmap( $context );
                break;
            default:
                $context = self::rmpDefault();
                roadmap_menu( $context );
                break;
        }
    }

    private function rmpDefault( $itemid = 0 ) {
    	$info = [
    	    'tablename' => self::$locale['RMP_010'],
    	    'sidename'  => self::$locale['RMP_S'.$itemid],
    	    'menu'      => self::RoadmapMenu(),
    	    'submit'    => ( ( iMEMBER && $this->rmpSettings['roadmap_submission'] && $this->rmpSettings['roadmap_access'] ) ? TRUE : FALSE ),
    	];
        if ( check_get( 'item_id' ) && get( 'item_id', FILTER_VALIDATE_INT ) ) {
            switch ( $this->action ) {
                case 'view':
                    if ( !check_get( 'rowstart' ) ) {
                        dbquery( "UPDATE " . DB_ROADMAP . " SET item_reads = item_reads+1 WHERE item_id = '" . get( 'item_id' ) . "'" );
                    }
                    $info['view'] = self::rmpView( get( 'item_id', FILTER_VALIDATE_INT ) );
                    break;
                case 'edit':
                    $info['edit'] = self::rmpEdit( get( 'item_id', FILTER_VALIDATE_INT ) );
                    break;
                case 'delete':
                    if ( iADMIN && $this->action == 'delete' && get( 'item_id', FILTER_VALIDATE_INT ) ) {
                        $result = dbquery("SELECT item_download FROM ".DB_DOWNLOADS." WHERE item_id = '" . get( 'item_id' ) . "'" );
                        if ( dbrows( $result ) ) {
                            if ( !empty( $data['item_download'] ) && file_exists( ROADMAP_FILES . $data['item_download'] ) ) {
                                @unlink( ROADMAP_FILES . $data['item_download'] );
                            }
                            dbquery( "DELETE FROM " . DB_ROADMAP . " WHERE item_id = :itemid", [':itemid' => (int)get( 'item_id' )] );
                            dbquery( "DELETE FROM " . DB_ROADMAP_HISTORY . " WHERE history_item_id = :itemid", [':itemid' => (int)get( 'item_id' )] );
                            dbquery( "DELETE FROM " . DB_COMMENTS . " WHERE comment_item_id = :commentid AND comment_type = :commenttype", [':commentid' => (int)get( 'item_id' ), ':commenttype' => 'RMP'] );
                            addnotice( 'success', self::$locale['RMP_012'] );
                        }
                        redirect( ROADMAP_PATH . 'roadmap.php' );
                    }
                    break;
                default:
                    break;
            }
        } else {
            $info['rmdata'] = self::defdata( $itemid );
        }
        //print_p($info);
    	return $info;
    }

    private function rmpEdit( $itemid = 0 ) {
        $result = self::defViewData( $itemid );
        $data = dbarray( $result );
        if ( check_post( 'item_save' ) ) {

            $savedat = [
                'item_id'          => sanitizer( 'item_id', '', 'item_id' ),
                'item_subject'     => sanitizer( 'item_subject', '', 'item_subject' ),
                'item_description' => sanitizer( 'item_description', '', 'item_description' ),
                'item_category'    => sanitizer( 'item_category', 0, 'item_category' ),
                'item_type'        => sanitizer( 'item_type', 0, 'item_type' ),
                'item_priority'    => sanitizer( 'item_priority', 0, 'item_priority' ),
                'item_version'     => sanitizer( 'item_version', '1.0', 'item_version' ),
                'item_assigned'    => sanitizer( 'item_assigned', 0, 'item_assigned' ),
                'item_access'      => sanitizer( 'item_access', 0, 'item_access' ),
                'item_download'    => sanitizer( 'item_download', '', 'item_download' ),
                'item_status'      => sanitizer( 'item_status', 0, 'item_status' ),
                'item_progress'    => sanitizer( 'item_progress', 0, 'item_progress' ),
                'item_published'   => sanitizer( 'item_published', 0, 'item_published' )
            ];
            if ( fusion_safe() ) {
                $savedat['item_note'] = self::addHistory( $data, $savedat ) . $data['item_note'];
                $savedat['item_updated'] = time();
                dbquery_insert( DB_ROADMAP, $savedat, 'update' );
                addnotice( "success", self::$locale['RMP_013'] );
                redirect( ROADMAP_PATH . 'roadmap.php?section=' . $this->sections );
            }
        }

        $info = [
            'opentable'        => $data['item_description'],
            'profile'          => profile_link( $data['user_id'], $data['user_name'], $data['user_status'] ),
            'datestamp'        => showdate( "Y-m.d", $data['item_datestamp'] ),
            'openform'         => openform( 'roadmapFrm', 'POST', FUSION_REQUEST ),
            'closeform'        => closeform(),
            'item_id'          => form_hidden( 'item_id', '', $data['item_id'] ),
            'item_subject'     => form_text( 'item_subject', self::$locale['RMP_014'], $data['item_subject'], [
                'inline'   => TRUE,
                'required' => TRUE
            ] ),
            'item_version'     => form_text( 'item_version', self::$locale['RMP_015'], $data['item_version'], [
                'inline'   => TRUE,
                'required' => TRUE
            ] ),
            'item_assigned'    => form_select( 'item_assigned', self::$locale['RMP_016'], $data['item_assigned'], [
                'inline'  => TRUE,
                'options' => self::RmpAsigUser()
            ] ),
            'item_category'    => form_select( 'item_category', self::$locale['RMP_017'], $data['item_category'], [
                'inline'       => TRUE,
                'options'      => $this->RmpSelectCat(),
                'parent_value' => self::$locale['choose']
            ] ),
            'item_type'        => form_select( 'item_type', self::$locale['RMP_018'], $data['item_type'], [
                'inline'       => TRUE,
                'options'      => $this->RmpSelectType(),
                'parent_value' => self::$locale['choose']
            ] ),
            'item_priority'    => form_select( 'item_priority', self::$locale['RMP_019'], $data['item_priority'], [
                'options'      => $this->RmpSelectPrior(),
                'inline'       => TRUE,
                'parent_value' => self::$locale['choose']
            ] ),
            'item_status'      => form_select( 'item_status', self::$locale['status'], $data['item_status'], [
                'options'      => $this->RmpSelectStatus(),
                'inline'       => TRUE,
                'parent_value' => self::$locale['choose']
            ] ),
            'item_progress'  => form_text( 'item_progress', self::$locale['RMP_020'], $data['item_progress'], [
                'inline'      => TRUE,
                'number'      => TRUE,
                'max_length'  => 4,
                'type'        => 'number',
                'inner_width' => '150px'
            ] ),
            'item_download'  => form_text( 'item_download', self::$locale['RMP_021'], $data['item_download'], [
                'inline' => TRUE,
                'type'   => 'url'
            ] ),
            'item_access'    => form_select( 'item_access', self::$locale['RMP_022'], $data['item_access'], [
                'inline'  => TRUE,
                'options' => fusion_get_groups()
            ] ),
            'item_description' => form_textarea( 'item_description', self::$locale['RMP_023'], $data['item_description'], [
                'inline'      => TRUE,
                'type'        => 'html',
                'autosize'    => TRUE,
                'placeholder' => self::$locale['RMP_024'],
                'form_name'   => 'roadmapFrm'
            ] ),
            'item_published'   => form_select( 'item_published', self::$locale['RMP_025'], $data['item_published'], [
                'inline'      => TRUE,
                'options'     => [0 => self::$locale['unpublish'], 1 => self::$locale['publish']],
                'placeholder' => self::$locale['choose']
            ] ),
            'savebtn'          => form_button( 'item_save', self::$locale['save'], self::$locale['save'], ['class' => 'btn-primary'] ),
        ];
        return $info;
    }

    private function rmpView( $itemid = NULL ) {
        $info = [];
        $result = self::defViewData( $itemid );
        if ( dbrows( $result ) ) {
            $data = dbarray( $result );
            $download = !empty( $data['item_download'] ) && checkgroup( $data['item_access'] ) && file_exists( ROADMAP_FILES . $data['item_download'] ) ? TRUE : FALSE;
            $info = [
                'link'           => ROADMAP_PATH . "roadmap.php?section=" . $this->sections . "&action=edit&item_id=" . $data['item_id'],
                'dellink'        => ROADMAP_PATH . "roadmap.php?section=" . $this->sections . "&action=delete&item_id=" . $data['item_id'],
                'avatar'         => self::RoadmapAvatar( display_avatar( $data, '30px', '', TRUE, 'img-rounded' ), profile_link( $data['user_id'], $data['user_name'], $data['user_status'] ) ),
                'datestamp'      => showdate( "Y-m.d", $data['item_datestamp'] ),
                'assigned'       => !empty( $data['item_assigned'] ) ? profile_link( $data['uid'], $data['uname'], $data['ustatus'] ) : self::$locale['RMP_026'],
                'version'        => $data['item_version'],
                'reads'          => $data['item_reads'],
                'description'    => parse_textarea( $data['item_description'], TRUE, TRUE, TRUE, FALSE, TRUE ),
                'note'           => parse_textarea( $data['item_note'], TRUE, TRUE, TRUE, FALSE, TRUE ),
                'subject'        => $data['item_subject'],
                'category'       => $this->RmpSelectCat( $data['item_category'] ),
                'priority'       => $this->RmpSelectPrior( $data['item_priority'] ),
                'status'         => $this->RmpSelectStatus( $data['item_status'] ),
                'type'           => $this->RmpSelectType( $data['item_type'] ),
                'progress'       => progress_bar( $data['item_progress'], '', ['hide_info' => TRUE, 'disabled' => TRUE] ),
                'csscolor'       => $this->tdclass[$data['item_status']],
                'download'       => !empty( $download ) ? ROADMAP_PATH . "roadmap.php?file_id=" . $data['item_id'] : '',
                'download_count' => $data['item_download_count'],
                'published'      => $data['item_published'] == 1 ? self::$locale['RMP_027'] : self::$locale['RMP_028'],
                'comments'       => self::getRoadmapComments( $data['item_id'] )
    	    ];
    	}
    	return $info;
    }

    private function getRoadmapComments( $itemid ) {
        $html = "";
        if ( fusion_get_settings( 'comments_enabled' ) ) {
            $html = \PHPFusion\Comments::getInstance( [
                'comment_item_type'     => 'RMP',
                'comment_db'            => DB_ROADMAP,
                'comment_col'           => 'item_id',
                'comment_item_id'       => $itemid,
                'clink'                 => ROADMAP_PATH . "roadmap.php?section=".$this->sections."&action=".$this->action."&item_id=" . $itemid,
                'comment_count'         => TRUE,
                'comment_allow_subject' => FALSE,
                'comment_allow_reply'   => TRUE,
                'comment_allow_post'    => TRUE,
                'comment_once'          => FALSE,
                'comment_allow_ratings' => FALSE,
            ], 'suggestion_comments' )->showComments();
        }
        return (string)$html;
    }

    private function addHistory( $old_data, $new_data ) {
        $sql = '';
        foreach( $new_data as $type => $value ) {
            if ( $value != $old_data[$type] ) {
                $sql .= "\n";
                $history_type = explode( "_", $type, 2 );
                $hist = array_flip( $this->history );
                $sql .= showdate( "Y-m.d", time() ) . ' - ' . $this->RmpSelectHistory( $hist[$history_type[1]] ) . self::RmpHistoryType( $hist[$history_type[1]], $old_data[$type], $value );
            }
        }
        return $sql;
    }
}

