<?php
namespace PHPFusion\Roadmap;

class RoadmapSubmit extends RoadmapServer {
    private static $instance = NULL;
    public $context = [];

    protected function __construct() {
        parent::__construct();
        add_to_title( self::$locale['RMP_040'] );
    }

    public static function getInstance() {
        if ( self::$instance == NULL ) {
            self::$instance = new static();
        }

        return self::$instance;
    }

    public function submitRoadmap() {
        if (iMEMBER && $this->rmpSettings['roadmap_submission'] && $this->rmpSettings['roadmap_access'] ) {
            $context = $this->displaySubmissionForm();
        } else {
            $context['no_submissions'] = self::$locale['RMP_041'];
        }
        roadmapSubmit( $context );
    }

    private function displaySubmissionForm() {
        if ( check_post( 'submit_roadmap' ) ) {
            $savedata = [
                'item_id'          => '',
                'item_user'        => fusion_get_userdata( 'user_id' ),
                'item_subject'     => sanitizer( 'item_subject', '', 'item_subject' ),
                'item_description' => sanitizer( 'item_description', '', 'item_description' ),
                'item_category'    => sanitizer( 'item_category', 0, 'item_category' ),
                'item_type'        => sanitizer( 'item_type', 0, 'item_type' ),
                'item_priority'    => sanitizer( 'item_priority', 0, 'item_priority' ),
                'item_version'     => 1,
                'item_assigned'    => 0,
                'item_status'      => 1,
                'item_datestamp'   => time(),
                'item_updated'     => 0,
                'item_language'    => sanitizer( ['item_language'], LANGUAGE, 'item_language' )
            ];
            if ( fusion_safe() ) {
                dbquery_insert( DB_ROADMAP, $savedata, 'save' );
                addnotice( "success", self::$locale['RMP_042'] );
                redirect( ROADMAP_PATH.'roadmap.php' );
            }
        }

        $info = [
            'opentable'        => self::$locale['RMP_043'],
            'subject'          => self::$locale['RMP_044'],
            'openform'         => openform( 'roadmapform', 'POST', FUSION_REQUEST ),
            'item_subject'     => form_text( 'item_subject', self::$locale['RMP_014'], '', [
                'inline'      => TRUE,
                'required'    => TRUE,
                'placeholder' => self::$locale['RMP_045']
            ] ),
            'item_type'        => form_select( 'item_type', self::$locale['RMP_018'], '', [
                'inline'       => TRUE,
                'required'     => TRUE,
                'options'      => $this->RmpSelectType(),
                'parent_value' => self::$locale['choose'],
                'max_select'   => 1,
                'multiple'     => TRUE,
                //'width'        => '100%',
                'inner_width'  => '100%'
            ] ),
            'item_category'    => form_select( 'item_category', self::$locale['RMP_017'], '', [
                'inline'       => TRUE,
                'required'     => TRUE,
                'options'      => $this->RmpSelectCat(),
                'parent_value' => self::$locale['choose'],
                'max_select'   => 1,
                'multiple'     => TRUE,
                'inner_width'  => '100%'
            ] ),
            'item_priority'    => form_select( 'item_priority', self::$locale['RMP_019'], '', [
                'inline'       => TRUE,
                'required'     => TRUE,
                'options'      => $this->RmpSelectPrior(),
                'parent_value' => self::$locale['choose'],
                'max_select'   => 1,
                'multiple'     => TRUE,
                'inner_width'  => '100%'
            ] ),
            'item_description' => form_textarea( 'item_description', self::$locale['RMP_023'], '', [
                'required'    => TRUE,
                'type'        => 'html',
                'autosize'    => TRUE,
                'placeholder' => self::$locale['RMP_024'],
                'form_name'   => 'roadmapform'
            ] ),
            'item_language'    => multilang_table( "RMP" ) ? form_select( 'item_language[]', self::$locale['language'], LANGUAGE, [
                'inline'      => TRUE,
                'options'     => fusion_get_enabled_languages(),
                'placeholder' => self::$locale['choose'],
                'inner_width' => '100%',
                'multiple'    => TRUE
            ] ) : form_hidden( 'item_language', '', LANGUAGE ),
            'button'           => form_button( 'submit_roadmap', self::$locale['submit'], self::$locale['submit'], ['class' => 'btn-primary'] ),
            'closeform'        => closeform()
        ];
        return $info;
    }
}
