<?php
/*-------------------------------------------------------+
| PHPFusion Content Management System
| Copyright (C) PHP Fusion Inc
| https://phpfusion.com/
+--------------------------------------------------------+
| Filename: classes/server.php
| Author: karrak
+--------------------------------------------------------+
| This program is released as free software under the
| Affero GPL license. You can redistribute it and/or
| modify it under the terms of this license which you
| can read by viewing the included agpl.txt or online
| at www.gnu.org/licenses/agpl.html. Removal of this
| copyright header is strictly prohibited without
| written permission from the original author(s).
+--------------------------------------------------------*/
namespace PHPFusion\Roadmap;

abstract class RoadmapServer {
    protected static $locale = [];
    public $sqlRoadmap = NULL;
    public $allowed_pages = ["empty", "new", "investigation", "rejected", "accepted", "development", "finished"];
    public $tdclass = ['', '', ' bg-warning', ' bg-danger', ' bg-primary', ' bg-info', ' bg-success'];
    public $history = ['', 'category', 'type', 'priority', 'assigned', 'status', 'dateils', 'version', 'progress', 'access', 'download', 'published', 'description'];
    public $action_pages = ['view', 'edit', 'delete'];
    public $rmpSettings;
    public $rpm_count;
    public $RmpSelectCat;
    public $rmpstatus;
    public $rmpprior;
    public $rmptype;
    public $rmphistory;
    public $rmplang;
    public $sections;
    public $action;

    public function __construct() {
        $this->sqlRoadmap = new RoadmapSql();
        add_to_title(\PHPFusion\SiteLinks::get_current_SiteLinks("", "link_name"));
    	require_once ROADMAP_PATH . 'templates/roadmap.tpl.php';
        self::RmpSetup();
        self::$locale = fusion_get_locale( '', ROADMAP_LOCALE );
        $this->sections = in_array(get('section'), $this->allowed_pages) ? get('section') : $this->allowed_pages[0];
        $this->action = in_array( get('action'), $this->action_pages) ? get('action') : '';
        $this->rmplang = multilang_table( "RMP" ) ? in_group( 'item_language', LANGUAGE ) : '';
    }

    public function RmpSetup() {
        $this->rmpSettings = get_settings( ROADMAP_FOLDER );
    }

    public function Rmpc() {
        $result = dbquery( $this->sqlRoadmap->getRpmQuery() );
        $this->rpm_count = dbarray( $result );
    }

    public function RmpAsigUser() {
        $result = dbquery( $this->sqlRoadmap->getAssigUserQuery() );
        $info[0] = self::$locale['RMP_030'];
        if ( dbrows( $result ) ) {
            while ( $data = dbarray( $result ) ) {
                $info[$data['user_id']] = $data['user_name'];
            }
        }
        return $info;
    }

    public function RmpHistoryType( $type = '', $old ='', $new = '' ) {
        switch ( $type ) {
            case 1:
                $text = sprintf( self::$locale['RMP_031'], $this->RmpSelectCat( $old ), $this->RmpSelectCat( $new ) );
            break;
            case 2:
                $text = sprintf( self::$locale['RMP_031'], $this->RmpSelectType( $old ), $this->RmpSelectType( $new ) );
            break;
            case 3:
                $text = sprintf( self::$locale['RMP_031'], $this->RmpSelectPrior( $old ), $this->RmpSelectPrior( $new ) );
            break;
            case 4:
                $old_user = FALSE;
                $new_user = FALSE;
                if ( $old != 0 ) {
                    $old_user = fusion_get_user( $old );
                    $assigned_old_profile = profile_link( $old_user['user_id'], $old_user['user_name'], $old_user['user_status'] );
                }
                if ( $new != 0 ) {
                    $new_user = fusion_get_user( $new );
                    $assigned_new_profile = profile_link( $new_user['user_id'], $new_user['user_name'], $new_user['user_status'] );
                }
                if ( $new_user && $old_user ) {
                    $text = sprintf( self::$locale['RMP_031'], $assigned_old_profile, $assigned_new_profile );
                } else if ( $new_user ) {
                    $text = sprintf( self::$locale['RMP_032'], $assigned_new_profile );
                } else {
                    $text = sprintf( self::$locale['RMP_033'], $assigned_old_profile );
                }
            break;
            case 5:
                $text = sprintf( self::$locale['RMP_031'], $this->RmpSelectStatus( $old ), $this->RmpSelectStatus( $new ) );
            break;
            case 6:
                $text = self::$locale['RMP_034'];
            break;
            case 7:
                $text = self::$locale['RMP_035'];
            break;
            case 8:
                $text = sprintf( self::$locale['RMP_031'], $old . '%', $new . '%' );
            break;
            case 9:
                $text = sprintf( self::$locale['RMP_031'], getgroupname( $old ), getgroupname(  $new ) );
            break;
            case 10:
                $text = self::$locale['RMP_036'];
            break;
            case 11:
                $pb = [0 => self::$locale['unpublish'], 1 => self::$locale['publish']];
                $text = sprintf( self::$locale['RMP_031'], $pb[$old], $pb[$new] );
            break;
            case 12:
                $text = self::$locale['RMP_037'];
            break;
            default:
                $text = sprintf( self::$locale['RMP_031'], $old, $new );
            break;

        }
        return $text;
	}

    public function defdata( $status = '' ) {
        $max_rows = dbcount( "(item_id)", DB_ROADMAP, $this->rmplang . ( $status ? " AND item_status = " . $status : '' ) );
        $rowstart = get_rowstart( "rowstart", $max_rows );
        $result = dbquery( $this->sqlRoadmap->getdefQuery( ['condition' => $this->rmplang. ( $status ? " AND item_status=" . $status : '' ), 'order' => 'item_updated DESC', 'limit' => $rowstart . "," . $this->rmpSettings['roadmap_pagination']] ) );
        $info = [];
        if ( dbrows( $result ) ) {
            while ( $data = dbarray( $result ) ) {
                $info['pagedata'][] = [
                    'sectionlink' => ROADMAP_PATH . "roadmap.php?section=" . $this->allowed_pages[$data['item_status']] . "&action=view&item_id=" . $data['item_id'],
                    'title'       => $data['item_subject'],
                    'count'       => $data['comments_count'],
                    'reads'       => $data['item_reads'],
                    'assigned'    => $data['item_assigned'] ? profile_link( $data['uid'], $data['uname'], $data['ustatus'] ) : self::$locale['RMP_026'],
                    'updated'     => $data['item_updated'] > $data['item_datestamp'] ? "<p>" . self::$locale['RMP_038'] . showdate("Y-m.d", $data['item_updated']) . "</p>" : '',
                    'datestamp'   => "<p>" . showdate( "Y-m.d", $data['item_datestamp'] ) . "</p>",
                    'category'    => $this->RmpSelectCat( $data['item_category'] ),
                    'csscolor'    => $this->tdclass[$data['item_status']],
                    'avatar'      => self::RoadmapAvatar( display_avatar( $data, '30px', '', TRUE, 'img-rounded' ), profile_link( $data['user_id'], $data['user_name'], $data['user_status'] ) )
                ];
            }
            $info['pagenav'] = makepagenav( $rowstart, $this->rmpSettings['roadmap_pagination'], $max_rows, 3, ROADMAP_PATH . "roadmap.php?section=".$this->allowed_pages[$status] . "&" );
        } else {
            $info['nopage'] = self::$locale['RMP_039'];

        }
        return $info;
	}

    public function defViewData( $itemid = 0 ) {
        return dbquery( $this->sqlRoadmap->getdefQuery( ['condition' => $this->rmplang . " AND item_id = " . $itemid, 'order' => 'item_updated DESC'] ) );
	}

    public function RmpSelectCat( $id = NULL ) {
        $text = [1 => self::$locale['RMP_C1'], 2 => self::$locale['RMP_C2'], 3 => self::$locale['RMP_C3'], 4 => self::$locale['RMP_C4'], 5 => self::$locale['RMP_C5'], 6 => self::$locale['RMP_C6'], 7 => self::$locale['RMP_C7'], 8 => self::$locale['RMP_C8'], 9 => self::$locale['RMP_C9'], 10 => self::$locale['RMP_C10']];
        return !empty($id) ? $text[$id] : $text;
	}

    public function RmpSelectStatus( $id = NULL ) {
        $text = [1 => self::$locale['RMP_S1'], 2 => self::$locale['RMP_S2'], 3 => self::$locale['RMP_S3'], 4 => self::$locale['RMP_S4'], 5 => self::$locale['RMP_S5'], 6 => self::$locale['RMP_S6'], 7 => self::$locale['RMP_S7'], 8 => self::$locale['RMP_S8']];
        return !empty($id) ? $text[$id] : $text;
	}

    public function RmpSelectPrior( $id = NULL ) {
        $text = [1 => self::$locale['RMP_P1'], 2 => self::$locale['RMP_P2'], 3 => self::$locale['RMP_P3'], 4 => self::$locale['RMP_P4']];
        return !empty($id) ? $text[$id] : $text;
	}

    public function RmpSelectType( $id = NULL ) {
        $text = [1 => self::$locale['RMP_T1'], 2 => self::$locale['RMP_T2'], 3 => self::$locale['RMP_T3']];
        return !empty($id) ? $text[$id] : $text;
	}

    public function RmpSelectHistory( $id = NULL ) {
        $text = [1 => self::$locale['RMP_H1'], 2 => self::$locale['RMP_H2'], 3 => self::$locale['RMP_H3'], 4 => self::$locale['RMP_H4'], 5 => self::$locale['RMP_H5'], 6 => self::$locale['RMP_H6'], 7 => self::$locale['RMP_H7'], 8 => self::$locale['RMP_H8'], 9 => self::$locale['RMP_H9'], 10 => self::$locale['RMP_H10'], 11 => self::$locale['RMP_H11'], 12 => self::$locale['RMP_H12']];
        return !empty($id) ? $text[$id] : $text;
	}

    public function RoadmapMenu() {
        self::Rmpc();
        $html = [];
        for ( $i=1; $i<=6; $i++ ) {
            $cellolor = $this->sections == $this->allowed_pages[$i] ? " bg-secondary" : $this->tdclass[$i];
            $textcolor = ( $i == 1 ? "text-dark" : "text-white" );
            $html[] = "<td class='text-center" . $cellolor . "'><a class='$textcolor' href = '" . ROADMAP_PATH . "roadmap.php?section=" . $this->allowed_pages[$i] . "'>" . self::$locale['RMP_S' . $i] . " (" . $this->rpm_count['status' . $i] . ")</a></td>";
        }
        return $html;
	}

	public function RoadmapAvatar( $avatar, $profile ) {
	    $text = "<div class='clearfix'>";
	        $text .= "<div class='pull-left m-r-10'>" . $avatar . "</div>";
	        $text .= "<div class='overflow-hide'><span class='m-l-10 m-r-10'>" . $profile . "</span></div>";
	     $text .= "</div>";
	     return $text;
	}
}
